//
//  test11.cpp
//  Lecture3CPP
//
//  Created by presentations on 11/3/17.
//
//

#include <iostream>
#include <cstring>
#include <cassert>
using namespace std;

class Employee {
public:
    Employee( const char*, const char* );  // constructor
    ~Employee();                                         // destructor
    const char *getFirstName() const;         // return first name
    const char *getLastName() const;         // return last name
    // static member function
    static int getCount();                              // return # objects instantiated
private:
    char *firstName;
    char *lastName;
    // static data member
    static int count;  // number of objects instantiated
}; // end class Employee (interface)

int Employee::count = 0; // Initialize the static data member
int Employee::getCount() { return count; } // static member function

Employee::Employee( const char *first, const char *last )
{
    firstName = new char[ strlen( first ) + 1 ];
    assert( firstName != 0 );   // ensure memory allocated
    strcpy( firstName, first );
    lastName = new char[ strlen( last ) + 1 ];
    assert( lastName != 0 );    // ensure memory allocated
    strcpy( lastName, last );
    ++count;  // increment static count of employees
    cout<<"Employee constructor for " << firstName << " " << lastName << " called" << endl;
} // end Employee constructor


// Destructor deallocates dynamically allocated memory
Employee::~Employee()
{
    cout << "~Employee() called for " << firstName << ' ' << lastName << endl;
    delete [ ] firstName;  // recapture memory
    delete [ ] lastName;   // recapture memory
    --count;                     // decrement static count of employees
} // end Employee destructor

const char *Employee::getFirstName() const
{
    // Const before return type prevents client from modifying
    // private data. Client should copy returned string before
    // destructor deletes storage to prevent undefined pointer.
    return firstName;
} // end function getFirstName

const char *Employee::getLastName() const
{
    return lastName;
} // end function getLastName
// end class Employee (implementation)

int main()
{                                                                                  // class name
    cout << "# of employees before instantiation is "<< Employee::getCount() << endl;
    Employee *e1Ptr = new Employee( "Susan", "Baker" );
    Employee *e2Ptr = new Employee( "Robert", "Jones" );
    cout << "#of employees after instantiation is " << e1Ptr->getCount();
    cout <<"\nEmployee 1:"<<e1Ptr->getFirstName()<<" "<< e1Ptr->getLastName() <<"\nEmployee 2:" <<e2Ptr->getFirstName()<<" "<<e2Ptr ->getLastName() << endl;
    delete e1Ptr;   // recapture memory
    e1Ptr = 0;       // disconnect e1Ptr from the previously allocated space
    delete e2Ptr;   // recapture memory
    e2Ptr = 0;                                                                 // class name
    cout<<"Number of employees after deletion is "<< Employee::getCount() << endl;
    return 0;
} // end function main

